# This script is designed to open access to ftp server for a certain amount of time
# after an irc user hits the trigger. It will also provide some statistics about
# the ftp server and the access list.

# The interface is implemented to the proftpd server - a secure, versatible ftp
# server for a *NIX environment.

# ChangeLog 
# 14-SEP-2002 : version 1.0 released

set ftp_username "ftpuser"
set ftp_password "ftppass"
set ftp_ip "127.0.0.1"
set ftp_port 21
set ftp_channel "#mychan"
set ftp_max_users 5
set ftp_bandwidth "128k"
set initial_ad_delay 2
set ad_delay 15
set ad_msg "TURN OFF PASV mode. Use only one connection"

if {![info exists {timer_already_set}]} {
        set timer_already_set 1
  	timer $initial_ad_delay adv_ftp
}

proc adv_ftp {} { 
	global ftp_channel ftp_ip ftp_port ftp_username ftp_password ftp_max_users ftp_bandwidth ad_msg ad_delay botnick

	putserv "PRIVMSG $ftp_channel : \002FTP Online\002 $ftp_ip:$ftp_port \002l/p\002:$ftp_username/$ftp_password \002Max users:\002$ftp_max_users \002Bandwidth:\002$ftp_bandwidth \002MOTD:\002$ad_msg. Use \00311/ctcp $botnick !LetMeIn\003 to get access. Other triggers \00311/ctcp $botnick !WhoIsOn\003 | \00311/ctcp $botnick !Allowed\003 | \00311/ctcp $botnick !History\003"

	timer $ad_delay adv_ftp
}

bind ctcp - !WhoIsOn msg_who

proc msg_who {nick uhost hand channel args} {
	global ftp_username

	putlog "WHO request from $nick has been receved!"
	if { [catch {set msg [exec -- /usr/bin/ftpwho -v | /bin/grep $ftp_username -A1 | /bin/grep "(host:"]} ]} {
    		putserv "NOTICE $nick :No hosts are connected"
	} else {
	        putserv "NOTICE $nick :The following hosts are connected with user $ftp_username"
        	set msg [split $msg "\n"]
	        foreach line $msg { putserv "NOTICE $nick :$line" }
	        putserv "NOTICE $nick :End of list"
	}
	return 0
}

bind ctcp - !Allowed msg_allowed

proc msg_allowed {nick uhost hand channel args} {
	global ftp_username

        putlog "ALLOWED request from $nick has been receved!"
	if { [catch {set msg [exec -- /bin/grep /etc/proftpd.allow -e "Allow "]} ]} {
		putserv "NOTICE $nick :No hosts are allowed to connect"
	} else {
	        putserv "NOTICE $nick :The following hosts are allowed to connect with user $ftp_username"
	        set msg [split $msg "\n"]
		foreach line $msg { putserv "NOTICE $nick :$line" }
		putserv "NOTICE $nick :End of list"
	}
        return 0
}

bind ctcp - !History msg_history

proc msg_history {nick uhost hand channel args} {
	global ftp_username

	putlog "HISTORY request from $nick has been receved!"
	if { [catch {set msg [exec -- /bin/grep /home/eggdrop/proftpd-prev/uniq -e "Allow "]} ]} {
		putserv "NOTICE $nick :No hosts were allowed to connect with user $ftp_username in the last 20 hours"
	} else {
		putserv "NOTICE $nick :The following hosts were allowed to connect with user $ftp_username in the last 24h"
		set msg [split $msg "\n"]
		foreach line $msg { putserv "NOTICE $nick :$line" }
		putserv "NOTICE $nick :End of list"
	}
        return 0
}

bind ctcp - !LetMeIn msg_letmein

proc msg_letmein {nick uhost hand channel args} {
	global ftp_channel
        if {[onchan $nick $ftp_channel] == 0} {
                putserv "NOTICE $nick :You are not on the required channel"
		putlog "Denied access attempt for $nick"
        } else {
		set host_name [lindex [split [lindex [split $uhost "@"] 1] ] 0]
	        set cur_hour [strftime %H]
	        putserv "NOTICE $nick :Allowing access to host $host_name"
		putserv "NOTICE $nick :Your ip is authorised to access for the next 6 hours"
		exec -- /bin/echo Allow $host_name # $nick >> /home/eggdrop/proftpd/hour$cur_hour
		exec -- /bin/touch /home/eggdrop/proftpd/trigger
		
		# the following part is for standalone proftpd. 
		# will have to wait for minutely crontab to act
	        set cur_sec [expr 60 - [strftime %S]]
	        putserv "NOTICE $nick :Please wait $cur_sec seconds for the change to become active"
	        
	        # the following part is for xinetd proftpd
	        # no need to wait fot crontab, since no root access needed, do the update yourself
	        # exec -- /path/to/update_ftp_access
	}
        return 0
}
